@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2019,2020 Thomas E. Dickey                                *
+ * Copyright 2018-2020,2022 Thomas E. Dickey                                *
  * Copyright 2017 Free Software Foundation, Inc.                            *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -30,13 +30,13 @@
 /*
  * Author: Thomas E. Dickey
  *
- * $Id: dots_xcurses.c,v 1.19 2020/02/02 23:34:34 tom Exp $
+ * $Id: dots_xcurses.c,v 1.25 2022/04/16 18:27:24 tom Exp $
  *
  * A simple demo of the wide-curses interface used for comparison with termcap.
  */
 #include <test.priv.h>
 
-#if !defined(_WIN32)
+#if !defined(_NC_WINDOWS)
 #include <sys/time.h>
 #endif
 
@@ -65,9 +65,10 @@ cleanup(void)
 {
     endwin();
 
-    printf("\n\n%ld total cells, rate %.2f/sec\n",
-	   total_chars,
-	   ((double) (total_chars) / (double) (time((time_t *) 0) - started)));
+    fflush(stdout);
+    fprintf(stderr, "\n\n%ld total cells, rate %.2f/sec\n",
+	    total_chars,
+	    ((double) (total_chars) / (double) (time((time_t *) 0) - started)));
 }
 
 static void
@@ -124,6 +125,7 @@ usage(void)
 	," -e       allow environment $LINES / $COLUMNS"
 #endif
 	," -m SIZE  set margin (default: 2)"
+	," -r SECS  self-interrupt/exit after specified number of seconds"
 	," -s MSECS delay 1% of the time (default: 1 msecs)"
 #if HAVE_ALLOC_PAIR
 	," -x       use alloc_pair() rather than init_pair()"
@@ -148,11 +150,12 @@ main(int argc, char *argv[])
     bool d_option = FALSE;
 #endif
     int m_option = 2;
+    int r_option = 0;
     int s_option = 1;
     size_t need;
     char *my_env;
 
-    while ((ch = getopt(argc, argv, "T:dem:s:x")) != -1) {
+    while ((ch = getopt(argc, argv, "T:dem:r:s:x")) != -1) {
 	switch (ch) {
 	case 'T':
 	    need = 6 + strlen(optarg);
@@ -173,6 +176,9 @@ main(int argc, char *argv[])
 	case 'm':
 	    m_option = atoi(optarg);
 	    break;
+	case 'r':
+	    r_option = atoi(optarg);
+	    break;
 	case 's':
 	    s_option = atoi(optarg);
 	    break;
@@ -187,8 +193,10 @@ main(int argc, char *argv[])
 	}
     }
 
+    setlocale(LC_ALL, "");
     srand((unsigned) time(0));
 
+    SetupAlarm(r_option);
     InitAndCatch(initscr(), onsig);
     if (has_colors()) {
 	start_color();
@@ -237,7 +245,8 @@ main(int argc, char *argv[])
 		set_colors(fg = z, bg);
 	    } else {
 		set_colors(fg, bg = z);
-		napms(s_option);
+		if (s_option)
+		    napms(s_option);
 	    }
 	} else {
 	    if (ranf() <= 0.01) {
@@ -246,7 +255,8 @@ main(int argc, char *argv[])
 		} else {
 		    attr_off(WA_REVERSE, NULL);
 		}
-		napms(s_option);
+		if (s_option)
+		    napms(s_option);
 	    }
 	}
 	wch[0] = (wchar_t) p;
